@@ -27,12 +27,7 @@
 #include "xf86str.h"
 #include "shadowfb.h"
 
-#ifdef RENDER
 # include "picturestr.h"
-#endif
-
-#define MIN(a,b) (((a)<(b))?(a):(b))
-#define MAX(a,b) (((a)>(b))?(a):(b))
 
 static Bool ShadowCloseScreen (int i, ScreenPtr pScreen);
 static void ShadowCopyWindow(
@@ -54,7 +49,6 @@ static Bool ShadowModifyPixmapHeader(
 static Bool ShadowEnterVT(int index, int flags);
 static void ShadowLeaveVT(int index, int flags);
 
-#ifdef RENDER
 static void ShadowComposite(
     CARD8 op,
     PicturePtr pSrc,
@@ -69,7 +63,6 @@ static void ShadowComposite(
     CARD16 width,
     CARD16 height
 );
-#endif /* RENDER */
 
 
 typedef struct {
@@ -80,9 +73,7 @@ typedef struct {
   CopyWindowProcPtr			CopyWindow;
   CreateGCProcPtr			CreateGC;
   ModifyPixmapHeaderProcPtr		ModifyPixmapHeader;
-#ifdef RENDER
   CompositeProcPtr Composite;
-#endif /* RENDER */
   Bool				(*EnterVT)(int, int);
   void				(*LeaveVT)(int, int);
   Bool				vtSema;
@@ -93,16 +84,16 @@ typedef struct {
    GCFuncs *funcs;
 } ShadowGCRec, *ShadowGCPtr;
 
+static DevPrivateKeyRec ShadowScreenKeyRec;
+#define ShadowScreenKey (&ShadowScreenKeyRec)
 
-static int ShadowScreenKeyIndex;
-static DevPrivateKey ShadowScreenKey = &ShadowScreenKeyIndex;
-static int ShadowGCKeyIndex;
-static DevPrivateKey ShadowGCKey = &ShadowGCKeyIndex;
+static DevPrivateKeyRec ShadowGCKeyRec;
+#define ShadowGCKey (&ShadowGCKeyRec)
 
 #define GET_SCREEN_PRIVATE(pScreen) \
     (ShadowScreenPtr)dixLookupPrivate(&(pScreen)->devPrivates, ShadowScreenKey)
 #define GET_GC_PRIVATE(pGC) \
-    (ShadowGCPtr)dixLookupPrivate(&(pGC)->devPrivates, ShadowGCKey);
+    (ShadowGCPtr)dixLookupPrivate(&(pGC)->devPrivates, ShadowGCKey)
 
 #define SHADOW_GC_FUNC_PROLOGUE(pGC)\
     ShadowGCPtr pGCPriv = GET_GC_PRIVATE(pGC);\
@@ -167,16 +158,17 @@ ShadowFBInit2 (
 ){
     ScrnInfoPtr pScrn = xf86Screens[pScreen->myNum];
     ShadowScreenPtr pPriv;
-#ifdef RENDER
     PictureScreenPtr ps = GetPictureScreenIfSet(pScreen);
-#endif /* RENDER */
 
     if(!preRefreshArea && !postRefreshArea) return FALSE;
     
-    if(!dixRequestPrivate(ShadowGCKey, sizeof(ShadowGCRec)))
+    if (!dixRegisterPrivateKey(&ShadowScreenKeyRec, PRIVATE_SCREEN, 0))
 	return FALSE;
 
-    if(!(pPriv = (ShadowScreenPtr)xalloc(sizeof(ShadowScreenRec))))
+    if(!dixRegisterPrivateKey(&ShadowGCKeyRec, PRIVATE_GC, sizeof(ShadowGCRec)))
+	return FALSE;
+
+    if(!(pPriv = (ShadowScreenPtr)malloc(sizeof(ShadowScreenRec))))
 	return FALSE;
 
     dixSetPrivate(&pScreen->devPrivates, ShadowScreenKey, pPriv);
@@ -202,12 +194,10 @@ ShadowFBInit2 (
     pScrn->EnterVT = ShadowEnterVT;
     pScrn->LeaveVT = ShadowLeaveVT;
 
-#ifdef RENDER
     if(ps) {
       pPriv->Composite = ps->Composite;
       ps->Composite = ShadowComposite;
     }
-#endif /* RENDER */
 
     return TRUE;
 }
@@ -226,9 +216,14 @@ static Bool
 ShadowEnterVT(int index, int flags)
 {
     ScrnInfoPtr pScrn = xf86Screens[index];
+    Bool ret;
     ShadowScreenPtr pPriv = GET_SCREEN_PRIVATE(pScrn->pScreen);
 
-    if((*pPriv->EnterVT)(index, flags)) {
+    pScrn->EnterVT = pPriv->EnterVT;
+    ret = (*pPriv->EnterVT)(index, flags);
+    pPriv->EnterVT = pScrn->EnterVT;
+    pScrn->EnterVT = ShadowEnterVT;
+    if(ret) {
 	pPriv->vtSema = TRUE;
         return TRUE;
     }
@@ -239,11 +234,15 @@ ShadowEnterVT(int index, int flags)
 static void
 ShadowLeaveVT(int index, int flags)
 {
+    ScrnInfoPtr pScrn = xf86Screens[index];
     ShadowScreenPtr pPriv = GET_SCREEN_PRIVATE(xf86Screens[index]->pScreen);
 
     pPriv->vtSema = FALSE;
 
+    pScrn->LeaveVT = pPriv->LeaveVT;
     (*pPriv->LeaveVT)(index, flags);
+    pPriv->LeaveVT = pScrn->LeaveVT;
+    pScrn->LeaveVT = ShadowLeaveVT;
 }
 
 /**********************************************************/
@@ -254,9 +253,7 @@ ShadowCloseScreen (int i, ScreenPtr pScreen)
 {
     ScrnInfoPtr pScrn = xf86Screens[pScreen->myNum];
     ShadowScreenPtr pPriv = GET_SCREEN_PRIVATE(pScreen);
-#ifdef RENDER
     PictureScreenPtr ps = GetPictureScreenIfSet(pScreen);
-#endif /* RENDER */
 
     pScreen->CloseScreen = pPriv->CloseScreen;
     pScreen->CopyWindow = pPriv->CopyWindow;
@@ -266,13 +263,11 @@ ShadowCloseScreen (int i, ScreenPtr pScreen)
     pScrn->EnterVT = pPriv->EnterVT;
     pScrn->LeaveVT = pPriv->LeaveVT;
 
-#ifdef RENDER
     if(ps) {
         ps->Composite = pPriv->Composite;
     }
-#endif /* RENDER */
 
-    xfree((pointer)pPriv);
+    free((pointer)pPriv);
 
     return (*pScreen->CloseScreen) (i, pScreen);
 }
@@ -289,18 +284,18 @@ ShadowCopyWindow(
     RegionRec rgnDst;
 
     if (pPriv->vtSema) {
-        REGION_NULL(pWin->drawable.pScreen, &rgnDst);
-	REGION_COPY(pWin->drawable.pScreen, &rgnDst, prgn);
+        RegionNull(&rgnDst);
+	RegionCopy(&rgnDst, prgn);
         
-        REGION_TRANSLATE(pWin->drawable.pScreen, &rgnDst,
+        RegionTranslate(&rgnDst,
                          pWin->drawable.x - ptOldOrg.x,
                          pWin->drawable.y - ptOldOrg.y);
-        REGION_INTERSECT(pScreen, &rgnDst, &pWin->borderClip, &rgnDst);
-        if ((num = REGION_NUM_RECTS(&rgnDst))) {
+        RegionIntersect(&rgnDst, &pWin->borderClip, &rgnDst);
+        if ((num = RegionNumRects(&rgnDst))) {
             if(pPriv->preRefresh)
-                (*pPriv->preRefresh)(pPriv->pScrn, num, REGION_RECTS(&rgnDst));
+                (*pPriv->preRefresh)(pPriv->pScrn, num, RegionRects(&rgnDst));
         } else {
-            REGION_UNINIT(pWin->drawable.pScreen, &rgnDst);
+            RegionUninit(&rgnDst);
         }
     }
     
@@ -310,8 +305,8 @@ ShadowCopyWindow(
     
     if (num) {
         if (pPriv->postRefresh)
-            (*pPriv->postRefresh)(pPriv->pScrn, num, REGION_RECTS(&rgnDst));
-        REGION_UNINIT(pWin->drawable.pScreen, &rgnDst);
+            (*pPriv->postRefresh)(pPriv->pScrn, num, RegionRects(&rgnDst));
+        RegionUninit(&rgnDst);
     }
 }
 
@@ -358,7 +353,6 @@ ShadowModifyPixmapHeader(
     return retval;
 }
 
-#ifdef RENDER
 static void
 ShadowComposite(
     CARD8 op,
@@ -411,7 +405,6 @@ ShadowComposite(
         (*pPriv->postRefresh)(pPriv->pScrn, 1, &box);
     }
 }
-#endif /* RENDER */
 
 /**********************************************************/
 
@@ -1029,7 +1022,7 @@ ShadowPolyRectangle(
 	    offset1 = offset2 >> 1;
 	    offset3 = offset2 - offset1;
 
-	    pBoxInit = (BoxPtr)xalloc(nRects * 4 * sizeof(BoxRec));
+	    pBoxInit = (BoxPtr)malloc(nRects * 4 * sizeof(BoxRec));
 	    pbox = pBoxInit;
 
 	    while(nRects--) {
@@ -1080,7 +1073,7 @@ ShadowPolyRectangle(
                 if(pPriv->preRefresh)
                     (*pPriv->preRefresh)(pPriv->pScrn, num, pBoxInit);
             } else {
-                xfree(pBoxInit);
+                free(pBoxInit);
             }                
 	}
     }
@@ -1092,7 +1085,7 @@ ShadowPolyRectangle(
     } else if(num) {
        if(pPriv->postRefresh)
           (*pPriv->postRefresh)(pPriv->pScrn, num, pBoxInit);
-       xfree(pBoxInit);
+       free(pBoxInit);
     }
     
     SHADOW_GC_OP_EPILOGUE(pGC);
@@ -1364,46 +1357,40 @@ ShadowFontToBox(BoxPtr BB, DrawablePtr pDrawable, GCPtr pGC, int x, int y,
     if (pFont->info.constantWidth) {
         int ascent, descent, left, right = 0;
 
-	ascent = MAX(pFont->info.fontAscent, pFont->info.maxbounds.ascent);
-	descent = MAX(pFont->info.fontDescent, pFont->info.maxbounds.descent);
+	ascent = max(pFont->info.fontAscent, pFont->info.maxbounds.ascent);
+	descent = max(pFont->info.fontDescent, pFont->info.maxbounds.descent);
 	left = pFont->info.maxbounds.leftSideBearing;
 	if (count > 0) {
 	    right = (count - 1) * pFont->info.maxbounds.characterWidth;
 	}
 	right += pFont->info.maxbounds.rightSideBearing;
 	BB->x1 =
-	    MAX(pDrawable->x + x - left, (REGION_EXTENTS(pGC->pScreen,
-		&((WindowPtr) pDrawable)->winSize))->x1);
+	    max(pDrawable->x + x - left,
+		RegionExtents(&((WindowPtr) pDrawable)->winSize)->x1);
 	BB->y1 =
-	    MAX(pDrawable->y + y - ascent,
-	    (REGION_EXTENTS(pGC->pScreen,
-             &((WindowPtr) pDrawable)->winSize))->y1);
+	    max(pDrawable->y + y - ascent,
+		RegionExtents(&((WindowPtr) pDrawable)->winSize)->y1);
 	BB->x2 =
-	    MIN(pDrawable->x + x + right,
-	    (REGION_EXTENTS(pGC->pScreen,
-             &((WindowPtr) pDrawable)->winSize))->x2);
+	    min(pDrawable->x + x + right,
+		RegionExtents(&((WindowPtr) pDrawable)->winSize)->x2);
 	BB->y2 =
-	    MIN(pDrawable->y + y + descent,
-	    (REGION_EXTENTS(pGC->pScreen,
-             &((WindowPtr) pDrawable)->winSize))->y2);
+	    min(pDrawable->y + y + descent,
+		RegionExtents(&((WindowPtr) pDrawable)->winSize)->y2);
     } else {
     	ShadowTextExtent(pFont, count, chars, wide ? (FONTLASTROW(pFont) == 0)
                          ? Linear16Bit : TwoD16Bit : Linear8Bit, BB);
 	BB->x1 =
-	    MAX(pDrawable->x + x + BB->x1, (REGION_EXTENTS(pGC->pScreen,
-		&((WindowPtr) pDrawable)->winSize))->x1);
+	    max(pDrawable->x + x + BB->x1,
+		RegionExtents(&((WindowPtr) pDrawable)->winSize)->x1);
 	BB->y1 =
-	    MAX(pDrawable->y + y + BB->y1,
-	    (REGION_EXTENTS(pGC->pScreen,
-             &((WindowPtr) pDrawable)->winSize))->y1);
+	    max(pDrawable->y + y + BB->y1,
+		RegionExtents(&((WindowPtr) pDrawable)->winSize)->y1);
 	BB->x2 =
-	    MIN(pDrawable->x + x + BB->x2,
-	    (REGION_EXTENTS(pGC->pScreen,
-	     &((WindowPtr) pDrawable)->winSize))->x2);
+	    min(pDrawable->x + x + BB->x2,
+		RegionExtents(&((WindowPtr) pDrawable)->winSize)->x2);
 	BB->y2 =
-	    MIN(pDrawable->y + y + BB->y2,
-	    (REGION_EXTENTS(pGC->pScreen, 
-	     &((WindowPtr) pDrawable)->winSize))->y2);
+	    min(pDrawable->y + y + BB->y2,
+		RegionExtents(&((WindowPtr) pDrawable)->winSize)->y2);
     }
 }
 
